import sys
import math
import os
import shutil
import tempfile
import pytest

import numpy as np
import numpy.ma as ma
import numpy.testing as npt

from astropy import wcs
from astropy.io import fits

from drizzle import drizzle

TEST_DIR = os.path.abspath(os.path.dirname(__file__))
DATA_DIR = os.path.join(TEST_DIR, 'data')
OUTPUT_DIR = os.environ.get('DRIZZLE_TEST_OUTPUT_DIR', tempfile.mkdtemp())

@pytest.yield_fixture(autouse=True, scope='module')
def output_dir():
    yield
    if 'DRIZZLE_TEST_OUTPUT_DIR' not in os.environ:
        shutil.rmtree(OUTPUT_DIR)

ok = False

def bound_image(image):
    """
    Compute region where image is non-zero
    """
    coords = np.nonzero(image)
    ymin = coords[0].min()
    ymax = coords[0].max()
    xmin = coords[1].min()
    xmax = coords[1].max()
    return (ymin, ymax, xmin, xmax)

def centroid(image, size, center):
    """
    Compute the centroid of a rectangular area
    """
    ylo = int(center[0] - size / 2)
    yhi = min(ylo + size, image.shape[0])
    xlo = int(center[1] - size / 2)
    xhi = min(xlo + size, image.shape[1])

    center = [0.0, 0.0, 0.0]
    for y in range(ylo, yhi):
        for x in range(xlo, xhi):
            center[0] += y * image[y,x]
            center[1] += x * image[y,x]
            center[2] += image[y,x]

    if center[2] == 0.0: return None

    center[0] /= center[2]
    center[1] /= center[2]
    return center

def centroid_close(list_of_centroids, size, point):
    """
    Find if any centroid is close to a point
    """
    for i in range(len(list_of_centroids)-1, -1, -1):
        if (abs(list_of_centroids[i][0] - point[0]) < int(size / 2) and
            abs(list_of_centroids[i][1] - point[1]) < int(size / 2)):
            return 1

    return 0

def centroid_compare(centroid):
    return centroid[1]

def centroid_distances(image1, image2, amp, size):
    """
    Compute a list of centroids and the distances between them in two images
    """
    distances = []
    list_of_centroids = centroid_list(image2, amp, size)
    for center2 in list_of_centroids:
        center1 = centroid(image1, size, center2)
        if center1 is None: continue

        disty = center2[0] - center1[0]
        distx = center2[1] - center1[1]
        dist = math.sqrt(disty * disty + distx * distx)
        dflux = abs(center2[2] - center1[2])
        distances.append([dist, dflux, center1, center2])

    distances.sort(key=centroid_compare)
    return distances

def centroid_list(image, amp, size):
    """
    Find the next centroid
    """
    list_of_centroids = []
    points = np.transpose(np.nonzero(image > amp))
    for point in points:
        if not centroid_close(list_of_centroids, size, point):
            center = centroid(image, size, point)
            list_of_centroids.append(center)

    return list_of_centroids

def centroid_statistics(title, fname, image1, image2, amp, size):
    """
    write centroid statistics to compare differences btw two images
    """
    stats = ("minimum", "median", "maximum")
    images = (None, None, image1, image2)
    im_type = ("", "", "test", "reference")

    diff = []
    distances = centroid_distances(image1, image2, amp, size)
    indexes = (0, int(len(distances)/2), len(distances)-1)
    fd = open(fname, 'w')
    fd.write("*** %s ***\n" % title)

    if len(distances) == 0:
        diff = [0.0, 0.0, 0.0]
        fd.write("No matches!!\n")

    elif len(distances) == 1:
        diff = [distances[0][0], distances[0][0], distances[0][0]]

        fd.write("1 match\n")
        fd.write("distance = %f flux difference = %f\n" % (distances[0][0], distances[0][1]))

        for j in range(2, 4):
            ylo = int(distances[0][j][0]) - 1
            yhi = int(distances[0][j][0]) + 2
            xlo = int(distances[0][j][1]) - 1
            xhi = int(distances[0][j][1]) + 2
            subimage = images[j][ylo:yhi,xlo:xhi]
            fd.write("\n%s image centroid = (%f,%f) image flux = %f\n" %
                     (im_type[j], distances[0][j][0], distances[0][j][1], distances[0][j][2]))
            fd.write(str(subimage) + "\n")

    else:
        fd.write("%d matches\n" % len(distances))

        for k in range(0,3):
            i = indexes[k]
            diff.append(distances[i][0])
            fd.write("\n%s distance = %f flux difference = %f\n" % (stats[k], distances[i][0], distances[i][1]))

            for j in range(2, 4):
                ylo = int(distances[i][j][0]) - 1
                yhi = int(distances[i][j][0]) + 2
                xlo = int(distances[i][j][1]) - 1
                xhi = int(distances[i][j][1]) + 2
                subimage = images[j][ylo:yhi,xlo:xhi]
                fd.write("\n%s %s image centroid = (%f,%f) image flux = %f\n" %
                         (stats[k], im_type[j], distances[i][j][0], distances[i][j][1], distances[i][j][2]))
                fd.write(str(subimage) + "\n")

    fd.close()
    return tuple(diff)

def make_point_image(input_image, point, value):
    """
    Create an image with a single point set
    """
    output_image = np.zeros(input_image.shape, dtype=input_image.dtype)
    output_image[point] = value
    return output_image

def make_grid_image(input_image, spacing, value):
    """
    Create an image with points on a grid set
    """
    output_image = np.zeros(input_image.shape, dtype=input_image.dtype)

    shape = output_image.shape
    half_space = int(spacing/2)
    for y in range(half_space, shape[0], spacing):
        for x in range(half_space, shape[1], spacing):
            output_image[y,x] = value

    return output_image

def print_wcs(title, wcs):
    """
    Print the wcs header cards
    """
    print("=== %s ===" % title)
    print(wcs.to_header_string())

def read_image(filename):
    """
    Read the image from a fits file
    """
    path = os.path.join(DATA_DIR, filename)
    hdu = fits.open(path)

    image = hdu[1].data
    hdu.close()
    return image

def read_wcs(filename):
    """
    Read the wcs of a fits file
    """
    path = os.path.join(DATA_DIR, filename)
    hdu = fits.open(path)
    the_wcs = wcs.WCS(hdu[1].header)
    hdu.close()
    return the_wcs

def test_square_with_point():
    """
    Test do_driz square kernel with point
    """
    input_file = os.path.join(DATA_DIR, 'j8bt06nyq_flt.fits')
    output = os.path.join(OUTPUT_DIR, 'output_square_point.fits')
    output_difference = os.path.join(OUTPUT_DIR, 'difference_square_point.txt')
    output_template = os.path.join(DATA_DIR, 'reference_square_point.fits')

    insci = read_image(input_file)
    inwcs = read_wcs(input_file)
    insci = make_point_image(insci, (500, 200), 100.0)
    inwht = np.ones(insci.shape,dtype=insci.dtype)
    output_wcs = read_wcs(output_template)

    driz = drizzle.Drizzle(outwcs=output_wcs, wt_scl="")
    driz.add_image(insci, inwcs, inwht=inwht)

    if ok:
        driz.write(output_template)
    else:
        driz.write(output)
        template_data = read_image(output_template)

        (min_diff, med_diff, max_diff) = centroid_statistics("square with point", output_difference,
                                                                  driz.outsci, template_data, 20.0, 8)

        assert(med_diff < 1.0e-6)
        assert(max_diff < 1.0e-5)

def test_square_with_grid():
    """
    Test do_driz square kernel with grid
    """
    input_file = os.path.join(DATA_DIR, 'j8bt06nyq_flt.fits')
    output = os.path.join(OUTPUT_DIR, 'output_square_grid.fits')
    output_difference = os.path.join(OUTPUT_DIR, 'difference_square_grid.txt')
    output_template = os.path.join(DATA_DIR, 'reference_square_grid.fits')

    insci = read_image(input_file)
    inwcs = read_wcs(input_file)
    insci = make_grid_image(insci, 64, 100.0)
    inwht = np.ones(insci.shape,dtype=insci.dtype)
    output_wcs = read_wcs(output_template)

    driz = drizzle.Drizzle(outwcs=output_wcs, wt_scl="")
    driz.add_image(insci, inwcs, inwht=inwht)

    if ok:
        driz.write(output_template)
    else:
        driz.write(output)
        template_data = read_image(output_template)

        (min_diff, med_diff, max_diff) = centroid_statistics("square with grid", output_difference,
                                                                  driz.outsci, template_data, 20.0, 8)

        assert(med_diff < 1.0e-6)
        assert(max_diff < 1.0e-5)

def test_turbo_with_grid():
    """
    Test do_driz turbo kernel with grid
    """
    input_file = os.path.join(DATA_DIR, 'j8bt06nyq_flt.fits')
    output = os.path.join(OUTPUT_DIR, 'output_turbo_grid.fits')
    output_difference = os.path.join(OUTPUT_DIR, 'difference_turbo_grid.txt')
    output_template = os.path.join(DATA_DIR, 'reference_turbo_grid.fits')

    insci = read_image(input_file)
    inwcs = read_wcs(input_file)
    insci = make_grid_image(insci, 64, 100.0)
    inwht = np.ones(insci.shape,dtype=insci.dtype)
    output_wcs = read_wcs(output_template)

    driz = drizzle.Drizzle(outwcs=output_wcs, wt_scl="", kernel='turbo')
    driz.add_image(insci, inwcs, inwht=inwht)

    if ok:
        driz.write(output_template)
    else:
        driz.write(output)
        template_data = read_image(output_template)

        (min_diff, med_diff, max_diff) = centroid_statistics("turbo with grid", output_difference,
                                                                  driz.outsci, template_data, 20.0, 8)

        assert(med_diff < 1.0e-6)
        assert(max_diff < 1.0e-5)

def test_gaussian_with_grid():
    """
    Test do_driz gaussian kernel with grid
    """
    input_file = os.path.join(DATA_DIR, 'j8bt06nyq_flt.fits')
    output = os.path.join(OUTPUT_DIR, 'output_gaussian_grid.fits')
    output_difference = os.path.join(OUTPUT_DIR, 'difference_gaussian_grid.txt')
    output_template = os.path.join(DATA_DIR, 'reference_gaussian_grid.fits')

    insci = read_image(input_file)
    inwcs = read_wcs(input_file)
    insci = make_grid_image(insci, 64, 100.0)
    inwht = np.ones(insci.shape,dtype=insci.dtype)
    output_wcs = read_wcs(output_template)

    driz = drizzle.Drizzle(outwcs=output_wcs, wt_scl="", kernel='gaussian')
    driz.add_image(insci, inwcs, inwht=inwht)

    if ok:
        driz.write(output_template)
    else:
        driz.write(output)
        template_data = read_image(output_template)

        (min_diff, med_diff, max_diff) = centroid_statistics("gaussian with grid", output_difference,
                                                                  driz.outsci, template_data, 20.0, 8)

        assert(med_diff < 1.0e-6)
        assert(max_diff < 2.0e-5)

def test_lanczos_with_grid():
    """
    Test do_driz lanczos kernel with grid
    """
    input_file = os.path.join(DATA_DIR, 'j8bt06nyq_flt.fits')
    output = os.path.join(OUTPUT_DIR, 'output_lanczos_grid.fits')
    output_difference = os.path.join(OUTPUT_DIR, 'difference_lanczos_grid.txt')
    output_template = os.path.join(DATA_DIR, 'reference_lanczos_grid.fits')

    insci = read_image(input_file)
    inwcs = read_wcs(input_file)
    insci = make_grid_image(insci, 64, 100.0)
    inwht = np.ones(insci.shape,dtype=insci.dtype)
    output_wcs = read_wcs(output_template)

    driz = drizzle.Drizzle(outwcs=output_wcs, wt_scl="", kernel='lanczos3')
    driz.add_image(insci, inwcs, inwht=inwht)

    if ok:
        driz.write(output_template)
    else:
        driz.write(output)
        template_data = read_image(output_template)

        (min_diff, med_diff, max_diff) = centroid_statistics("lanczos with grid", output_difference,
                                                                  driz.outsci, template_data, 20.0, 8)

        assert(med_diff < 1.0e-6)
        assert(max_diff < 1.0e-5)

def test_tophat_with_grid():
    """
    Test do_driz tophat kernel with grid
    """
    input_file = os.path.join(DATA_DIR, 'j8bt06nyq_flt.fits')
    output = os.path.join(OUTPUT_DIR, 'output_tophat_grid.fits')
    output_difference = os.path.join(OUTPUT_DIR, 'difference_tophat_grid.txt')
    output_template = os.path.join(DATA_DIR, 'reference_tophat_grid.fits')

    insci = read_image(input_file)
    inwcs = read_wcs(input_file)
    insci = make_grid_image(insci, 64, 100.0)
    inwht = np.ones(insci.shape,dtype=insci.dtype)
    output_wcs = read_wcs(output_template)

    driz = drizzle.Drizzle(outwcs=output_wcs, wt_scl="", kernel='tophat')
    driz.add_image(insci, inwcs, inwht=inwht)

    if ok:
        driz.write(output_template)
    else:
        driz.write(output)
        template_data = read_image(output_template)

        (min_diff, med_diff, max_diff) = centroid_statistics("tophat with grid", output_difference,
                                                                  driz.outsci, template_data, 20.0, 8)

        assert(med_diff < 1.0e-6)
        assert(max_diff < 1.0e-5)

def test_point_with_grid():
    """
    Test do_driz point kernel with grid
    """
    input_file = os.path.join(DATA_DIR, 'j8bt06nyq_flt.fits')
    output = os.path.join(OUTPUT_DIR, 'output_point_grid.fits')
    output_difference = os.path.join(OUTPUT_DIR, 'difference_point_grid.txt')
    output_template = os.path.join(DATA_DIR, 'reference_point_grid.fits')

    insci = read_image(input_file)
    inwcs = read_wcs(input_file)
    insci = make_grid_image(insci, 64, 100.0)
    inwht = np.ones(insci.shape,dtype=insci.dtype)
    output_wcs = read_wcs(output_template)

    driz = drizzle.Drizzle(outwcs=output_wcs, wt_scl="", kernel='point')
    driz.add_image(insci, inwcs, inwht=inwht)

    if ok:
        driz.write(output_template)
    else:
        driz.write(output)
        template_data = read_image(output_template)

        (min_diff, med_diff, max_diff) = centroid_statistics("point with grid", output_difference,
                                                                  driz.outsci, template_data, 20.0, 8)

        assert(med_diff < 1.0e-6)
        assert(max_diff < 1.0e-5)

def test_blot_with_point():
    """
    Test do_blot with point image
    """
    input_file = os.path.join(DATA_DIR, 'j8bt06nyq_flt.fits')
    output = os.path.join(OUTPUT_DIR, 'output_blot_point.fits')
    output_difference = os.path.join(OUTPUT_DIR, 'difference_blot_point.txt')
    output_template = os.path.join(DATA_DIR, 'reference_blot_point.fits')

    outsci = read_image(input_file)
    outwcs = read_wcs(input_file)
    outsci = make_point_image(outsci, (500, 200), 40.0)
    inwcs = read_wcs(output_template)

    driz = drizzle.Drizzle(outwcs=outwcs)
    driz.outsci = outsci

    driz.blot_image(inwcs)

    if ok:
        driz.write(output_template)
    else:
        driz.write(output)
        template_data = read_image(output_template)

        (min_diff, med_diff, max_diff) = centroid_statistics("blot with point", output_difference,
                                                                  driz.outsci, template_data, 20.0, 16)
        assert(med_diff < 1.0e-6)
        assert(max_diff < 1.0e-5)

def test_blot_with_default():
    """
    Test do_blot with default grid image
    """
    input_file = os.path.join(DATA_DIR, 'j8bt06nyq_flt.fits')
    output = os.path.join(OUTPUT_DIR, 'output_blot_default.fits')
    output_difference = os.path.join(OUTPUT_DIR, 'difference_blot_default.txt')
    output_template = os.path.join(DATA_DIR, 'reference_blot_default.fits')

    outsci = read_image(input_file)
    outsci = make_grid_image(outsci, 64, 100.0)
    outwcs = read_wcs(input_file)
    inwcs = read_wcs(output_template)

    driz = drizzle.Drizzle(outwcs=outwcs)
    driz.outsci = outsci

    driz.blot_image(inwcs)

    if ok:
        driz.write(output_template)
    else:
        driz.write(output)
        template_data = read_image(output_template)

        (min_diff, med_diff, max_diff) = centroid_statistics("blot with defaults", output_difference,
                                                                  driz.outsci, template_data, 20.0, 16)

        assert(med_diff < 1.0e-6)
        assert(max_diff < 1.0e-5)

def test_blot_with_lan3():
    """
    Test do_blot with lan3 grid image
    """
    input_file = os.path.join(DATA_DIR, 'j8bt06nyq_flt.fits')
    output = os.path.join(OUTPUT_DIR, 'output_blot_lan3.fits')
    output_difference = os.path.join(OUTPUT_DIR, 'difference_blot_lan3.txt')
    output_template = os.path.join(DATA_DIR, 'reference_blot_lan3.fits')

    outsci = read_image(input_file)
    outsci = make_grid_image(outsci, 64, 100.0)
    outwcs = read_wcs(input_file)
    inwcs = read_wcs(output_template)

    driz = drizzle.Drizzle(outwcs=outwcs)
    driz.outsci = outsci

    driz.blot_image(inwcs, interp="lan3")

    if ok:
        driz.write(output_template)
    else:
        driz.write(output)
        template_data = read_image(output_template)

        (min_diff, med_diff, max_diff) = centroid_statistics("blot with lan3", output_difference,
                                                                  driz.outsci, template_data, 20.0, 16)

        assert(med_diff < 1.0e-6)
        assert(max_diff < 1.0e-5)

def test_blot_with_lan5():
    """
    Test do_blot with lan5 grid image
    """
    input_file = os.path.join(DATA_DIR, 'j8bt06nyq_flt.fits')
    output = os.path.join(OUTPUT_DIR, 'output_blot_lan5.fits')
    output_difference = os.path.join(OUTPUT_DIR, 'difference_blot_lan5.txt')
    output_template = os.path.join(DATA_DIR, 'reference_blot_lan5.fits')

    outsci = read_image(input_file)
    outsci = make_grid_image(outsci, 64, 100.0)
    outwcs = read_wcs(input_file)
    inwcs = read_wcs(output_template)

    driz = drizzle.Drizzle(outwcs=outwcs)
    driz.outsci = outsci

    driz.blot_image(inwcs, interp="lan5")

    if ok:
        driz.write(output_template)
    else:
        driz.write(output)
        template_data = read_image(output_template)

        (min_diff, med_diff, max_diff) = centroid_statistics("blot with lan5", output_difference,
                                                                  driz.outsci, template_data, 20.0, 16)

        assert(med_diff < 1.0e-6)
        assert(max_diff < 1.0e-5)

if __name__ == "__main__":
    """
    Run tests from command line
    """
    for flag in sys.argv[1:]:
        if flag == 'ok':
            ok = True

    test_square_with_point()
    test_square_with_grid()
    test_turbo_with_grid()
    test_gaussian_with_grid()
    test_lanczos_with_grid()
    test_tophat_with_grid()
    test_point_with_grid()
    test_blot_with_point()
    test_blot_with_default()
    test_blot_with_lan3()
    test_blot_with_lan5()



import sys
import os.path
import numpy as np
import numpy.testing as npt

from astropy import wcs
from astropy.io import fits

from drizzle import calc_pixmap

TEST_DIR = os.path.abspath(os.path.dirname(__file__))
DATA_DIR = os.path.join(TEST_DIR, 'data')

def test_map_rectangular():
    """
    Make sure the initial index array has correct values
    """
    naxis1 = 1000
    naxis2 = 10

    pixmap = np.indices((naxis1, naxis2), dtype='float32')
    pixmap = pixmap.transpose()
    npt.assert_equal(pixmap[5,500], (500,5))

def test_map_to_self():
    """
    Map a pixel array to itself. Should return the same array.
    """
    input_file = os.path.join(DATA_DIR, 'input1.fits')
    input_hdu = fits.open(input_file)
    input_header = input_hdu[1].header
    input_shape = input_hdu[1].data.shape

    input_wcs = wcs.WCS(input_hdu[1].header)
    naxis1, naxis2 = input_wcs.pixel_shape
    input_hdu.close()

    ok_pixmap = np.indices((naxis1, naxis2), dtype='float32')
    ok_pixmap = ok_pixmap.transpose()

    pixmap = calc_pixmap.calc_pixmap(input_wcs, input_wcs)
    npt.assert_equal(pixmap.shape, ok_pixmap.shape) # Got x-y transpose right
    npt.assert_almost_equal(pixmap, ok_pixmap, decimal=5) # Mapping an array to itself

def test_translated_map():
    """
    Map a pixel array to  at translated array.
    """
    first_file = os.path.join(DATA_DIR, 'input1.fits')
    first_hdu = fits.open(first_file)
    first_header = first_hdu[1].header

    first_wcs = wcs.WCS(first_header)
    naxis1, naxis2 = first_wcs.pixel_shape
    first_hdu.close()

    second_file = os.path.join(DATA_DIR, 'input3.fits')
    second_hdu = fits.open(second_file)
    second_header = second_hdu[1].header

    second_wcs = wcs.WCS(second_header)
    second_hdu.close()

    ok_pixmap = np.indices((naxis1, naxis2), dtype='float32') - 2.0
    ok_pixmap = ok_pixmap.transpose()

    pixmap = calc_pixmap.calc_pixmap(first_wcs, second_wcs)
    npt.assert_equal(pixmap.shape, ok_pixmap.shape) # Got x-y transpose right
    npt.assert_almost_equal(pixmap, ok_pixmap, decimal=5) # Mapping an array to a translated array



import sys
import glob
import math
import os
import shutil
import tempfile
import pytest
import numpy as np
import numpy.ma as ma
import numpy.testing as npt

from astropy import wcs
from astropy.io import fits

from drizzle import drizzle
from drizzle import util

TEST_DIR = os.path.abspath(os.path.dirname(__file__))
DATA_DIR = os.path.join(TEST_DIR, 'data')
OUTPUT_DIR = os.environ.get('DRIZZLE_TEST_OUTPUT_DIR', tempfile.mkdtemp())

@pytest.yield_fixture(autouse=True, scope='module')
def output_dir():
    yield
    if 'DRIZZLE_TEST_OUTPUT_DIR' not in os.environ:
        shutil.rmtree(OUTPUT_DIR)

def read_header(filename):
    """
    Read the primary header from a fits file
    """
    fileroot, extn = util.parse_filename(os.path.join(DATA_DIR, filename))
    hdu = fits.open(fileroot)

    header = hdu[0].header
    hdu.close()
    return header

def read_image(filename):
    """
    Read the image from a fits file
    """
    fileroot, extn = util.parse_filename(os.path.join(DATA_DIR, filename))
    hdu = fits.open(fileroot)

    image = hdu[1].data
    hdu.close()
    return image

def read_wcs(filename):
    """
    Read the wcs of a fits file
    """
    fileroot, extn = util.parse_filename(os.path.join(DATA_DIR, filename))
    hdu = fits.open(fileroot)

    the_wcs = wcs.WCS(hdu[1].header)
    hdu.close()
    return the_wcs

def test_null_run():
    """
    Create an empty drizzle image
    """
    output_file = os.path.join(OUTPUT_DIR, 'output_null_run.fits')
    output_template = os.path.join(DATA_DIR, 'reference_square_point.fits')

    output_wcs = read_wcs(output_template)
    driz = drizzle.Drizzle(outwcs=output_wcs, wt_scl="expsq",
                                   pixfrac=0.5, kernel="turbo",
                                   fillval="NaN")
    driz.write(output_file)

    assert(os.path.exists(output_file))
    handle = fits.open(output_file)

    assert(handle.index_of("SCI") == 1)
    assert(handle.index_of("WHT") == 2)
    assert(handle.index_of("CTX") == 3)

    pheader = handle[0].header
    assert(pheader['DRIZOUDA'] == 'SCI')
    assert(pheader['DRIZOUWE'] == 'WHT')
    assert(pheader['DRIZOUCO'] == 'CTX')
    assert(pheader['DRIZWTSC'] == 'expsq')
    assert(pheader['DRIZKERN'] == 'turbo')
    assert(pheader['DRIZPIXF'] == 0.5)
    assert(pheader['DRIZFVAL'] == 'NaN')
    assert(pheader['DRIZOUUN'] == 'cps')
    assert(pheader['EXPTIME'] == 0.0)
    assert(pheader['DRIZEXPT'] == 1.0)

def test_file_init():
    """
    Initialize drizzle object from a file
    """
    input_file = os.path.join(OUTPUT_DIR, 'output_null_run.fits')
    output_file = os.path.join(OUTPUT_DIR, 'output_null_run.fits')

    driz = drizzle.Drizzle(infile=input_file)
    driz.write(output_file)

    assert(driz.outexptime == 1.0)
    assert(driz.wt_scl == 'expsq')
    assert(driz.kernel == 'turbo')
    assert(driz.pixfrac == 0.5)
    assert(driz.fillval == 'NaN')

def test_add_header():
    """
    Add extra keywords read from the header
    """
    input_file = os.path.join(DATA_DIR, 'j8bt06nyq_flt.fits')
    output_file = os.path.join(OUTPUT_DIR, 'output_add_header.fits')
    output_template = os.path.join(DATA_DIR, 'reference_square_point.fits')

    driz = drizzle.Drizzle(infile=output_template)
    image = read_image(input_file)
    inwcs = read_wcs(input_file)
    driz.add_image(image, inwcs)

    header = fits.header.Header()
    header['ONEVAL'] = (1.0, 'test value')
    header['TWOVAL'] = (2.0, 'test value')

    driz.write(output_file, outheader=header)

    header = read_header(output_file)
    assert(header['ONEVAL'] == 1.0)
    assert(header['TWOVAL'] == 2.0)
    assert(header['DRIZKERN'] == 'square')

def test_add_file():
    """
    Add an image read from a file
    """
    input_file = os.path.join(DATA_DIR, 'j8bt06nyq_flt.fits[1]')
    output_file = os.path.join(OUTPUT_DIR, 'output_add_file.fits')
    test_file = os.path.join(OUTPUT_DIR, 'output_add_header.fits')
    output_template = os.path.join(DATA_DIR, 'reference_square_point.fits')

    driz = drizzle.Drizzle(infile=output_template)
    driz.add_fits_file(input_file)
    driz.write(output_file)

    output_image = read_image(output_file)
    test_image =  read_image(test_file)
    diff_image = np.absolute(output_image - test_image)
    assert(np.amax(diff_image) == 0.0)

def test_blot_file():
    """
    Blot an image read from a file
    """
    input_file = os.path.join(DATA_DIR, 'j8bt06nyq_flt.fits[1]')
    output_file = os.path.join(OUTPUT_DIR, 'output_blot_file.fits')
    test_file = os.path.join(OUTPUT_DIR, 'output_blot_image.fits')
    output_template = os.path.join(DATA_DIR, 'reference_blot_image.fits')

    blotwcs = read_wcs(input_file)

    driz = drizzle.Drizzle(infile=output_template)
    driz.add_fits_file(input_file)
    driz.blot_image(blotwcs)
    driz.write(test_file)

    driz = drizzle.Drizzle(infile=output_template)
    driz.add_fits_file(input_file)
    driz.blot_fits_file(input_file)
    driz.write(output_file)

    output_image = read_image(output_file)
    test_image =  read_image(test_file)
    diff_image = np.absolute(output_image - test_image)
    assert(np.amax(diff_image) == 0.0)



#!/usr/bin/env python
# Licensed under a 3-clause BSD style license - see LICENSE.rst

import os
import sys

from setuptools import setup, find_packages, Extension
from configparser import ConfigParser


try:
    import numpy
except ImportError:
    print("numpy is required to build this package.", file=sys.stderr)
    print("HINT: execute \"pip install .\"", file=sys.stderr)
    exit(1)


def get_extensions():
    ROOT = os.path.dirname(__file__)
    SRCDIR = os.path.join(ROOT, 'src')
    cfg = {
        'include_dirs': [],
        'libraries': [],
        'define_macros': []
    }

    cdriz_sources = ['cdrizzleapi.c',
                     'cdrizzleblot.c',
                     'cdrizzlebox.c',
                     'cdrizzlemap.c',
                     'cdrizzleutil.c',
                     os.path.join('tests', 'utest_cdrizzle.c')]

    sources = [os.path.join(SRCDIR, x) for x in cdriz_sources]

    cfg['include_dirs'].append(numpy.get_include())
    cfg['include_dirs'].append(SRCDIR)

    if sys.platform != 'win32':
        cfg['libraries'].append('m')

    if sys.platform == 'win32':
        cfg['define_macros'].append(('WIN32', None))
        cfg['define_macros'].append(('__STDC__', 1))
        cfg['define_macros'].append(('_CRT_SECURE_NO_WARNINGS', None))

    return [Extension(str('drizzle.cdrizzle'), sources, **cfg)]


conf = ConfigParser()
conf.read(['setup.cfg'])
metadata = dict(conf.items('metadata'))

PACKAGENAME = metadata.get('package_name', 'packagename')
DESCRIPTION = metadata.get('description', 'Astropy affiliated package')
AUTHOR = metadata.get('author', '')
AUTHOR_EMAIL = metadata.get('author_email', '')
LICENSE = metadata.get('license', 'unknown')
URL = metadata.get('url', 'https://www.stsci.edu')

# Get the long description from the package's docstring
__import__(PACKAGENAME)
package = sys.modules[PACKAGENAME]
LONG_DESCRIPTION = package.__doc__

# Include all .c files, recursively, including those generated by
# Cython, since we can not do this in MANIFEST.in with a "dynamic"
# directory name.
def to_compile(package):
    c_files = []
    for root, dirs, files in os.walk(package):
        for filename in files:
            if filename.endswith('.c') or filename.endswith('.h'):
                c_files.append(
                    os.path.join(
                        os.path.relpath(root, package), filename))
    return c_files


TESTS_REQUIRE = [
    'pytest',
    'pytest-runner'
]

DOCS_REQUIRE = [
    'sphinx',
    'sphinx_automodapi',
    'matplotlib'
]

setup(name=PACKAGENAME,
    use_scm_version=True,
    description=DESCRIPTION,
    python_requires='>=3.5',
    setup_requires=['setuptools_scm'],
    install_requires=['Cython', 'astropy'],
    tests_require=TESTS_REQUIRE,
    extras_require={
        'test': TESTS_REQUIRE,
        'docs': DOCS_REQUIRE
    },
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license=LICENSE,
    url=URL,
    long_description=LONG_DESCRIPTION,
    zip_safe=False,
    packages=find_packages(),
    ext_modules=get_extensions(),
)
